'use strict';

import procnet from 'procnet';
import error from '../error';
import crypto from '../promise/crypto';
import R from 'ramda';
import express from 'expressed';
import csrfToken from '../csrf-token';

const cleanUser = R.pick(['name', 'email', 'id', 'active', 'roles']);

module.exports = procnet.injectable((
		mail, 
		withPerms, 
		userStore, 
		userValidation, 
		userActivationStore,
		sessionTokenCache) => {
	const router = express.Router();

	router.post('/register', (req, rest) => {
		const { name, email, password } = req.body;
		return userValidation
			.create(name, email, password)
			.then(() => {
				return userStore.create(name, email, password);
			})
			.then((user) => {
				return userActivationStore
					.create(user.id)
					.then((token) => {
						return mail.send({
							template: 'activate',
							locals: {
								token,
								name: user.name
							},
							to: email,
						});
					})
					.thenReturn(user);
			})
			.then(cleanUser);
	});

	router.post('/activate', (req, res) => {
		const { user_id, token } = req.body;
		return userActivationStore
			.verify(user_id, token)
			.then((isValidToken) => {
					if(!isValidToken) {
						return error('activation_token_missing');
					}
					return userStore
						.activate(user_id)
						.thenReturn({
							message: 'user_activated'
						});
				});
			
	});

	router.post('/login', (req, res) => {
		const { email, password } = req.body;

		return userStore
				.find('email', email)
				.then(userValidation.login(email, password))
				.spread((user, pwMatches) => {
					if(!pwMatches) {
						return userStore
							.failLogin(user.id)
							.thenReturn(error('password_invalid'));
					}

					return sessionTokenCache
						.create(user.id)
						.then((session_token) => {
							res.cookie('session', session_token);
							return R.merge({
								csrf: csrfToken(session_token)
							}, cleanUser(user));
						});
				});
	});

	router.get('/login', withPerms, (req, res) => {
		return userStore
			.find('id', req.userId)
			.then(cleanUser);
	});

	return router;
});

