'use strict';

/**
 * Express app loading
 */

import conf from './config';
import helmet from 'helmet';
import R from 'ramda';
import express from 'expressed';
import csrfToken from './csrf-token';
import cookieParser from 'cookie-parser';
import bodyParser from 'body-parser';
import error from './error';
import log from './log';

// This needs to be shared between all nodes
const secret = conf.security.secret;

const app = express();

app.use(cookieParser());
app.use(bodyParser.json());

app.use((req, res, next) => {
	// If the session token is set, I expect the csrf header
	// to be set with the correct value. The client is expected
	// to add this token on every request through javascript.
	// The session token cannot be acessed through js, therefore
	// making it even more difficult to hijack sessions.
	//
	// Even though all actions are JSON calls, its technically
	// possible to send a CSRF payload using `application/text`
	// content type on a form. Also possible there are ways
	// around making all actions REST calls, so minimizing the
	// attack surface is just better.
	const sessionToken = req.cookies.sessionToken;
	if(sessionToken) {
		const csrfToken = req.get('X-Csrf-Token');
		const rehash = csrfToken(sessionToken);

		if(rehash !== csrfToken) {
			res.writeHead(401);
			res.end('csrf_mismatch');
			return;
		}
	}
	next();
});

// Prevent web page from being added to an xframe.
app.use(helmet.frameguard());

// Don't want the mime type to be infered automatically.
app.use(helmet.noSniff());

// Help mitigate dns attacks.
app.use(helmet.dnsPrefetchControl());

// Limit sites which can be used for loading assets. Limited form of XSS
// protection. This will work in IE 10+
app.use(helmet.csp({
	directives: {
		// TODO: config?
		defaultSrc: ["'self'"]
	}
}));

// Sort of todo:
// Assets urls contain hash of the contents; allows to force updates.
const loadApp = (fn) => {
	fn(app);
	app.use((err, req, res, next) => {

		if(!error.isSystemError(err)) {
			log.error(err.stack);
			res.status(500).send('internal_server_error');
			return;
		}

		log.debug('error', err.stack);

		const code = err.code || 500;
		if(err.message) {
			res.status(code).send(err.message);
		} else {
			const message = {
				400: 'bad_request',
				500: 'internal_server_error'
			}[code];
			res.status(code).send(message || 'unknown_error');
		}
	});
	return app;
};

export default loadApp;

