'use strict';

/**
 * Cache for the role -> permissions mapping.
 */
const procnet = require('procnet');
const R = require('ramda');

// TODO: remove knex
export default procnet.injectable((knex, cacheEventBus) => {
	const cache = {};

	const insertRole = (trx, role, perms) => {
		const rows = perms.map((permission) => {
			return {
				role,
				permission
			};
		});

		return trx
			.insert(rows)
			.into('role_permission')
			.then(() => {
				cacheEventBus.publish({
					type: 'role',
					event: 'update',
					role,
					perms
				});
			});
	};

	const replaceRole = (trx, role, perms) => {
		return trx
			.del()
			.from('role_permission')
			.where('role', role)
			.then(() => insertRole(trx, role, perms));
	};

	cacheEventBus.on('update:role', (msg) => {
		cache[msg.role] = msg.perms;
	});

	cacheEventBus.on('remove:role', (msg) => {
		delete cache[msg.role];
	});
	

	/**
	 * This method is called on bootstrap to load the state from the database
	 * and initialize the event bus connection.
	 */
	const loadModule = () => {
		return knex('role_permission')
			.select()
			.then((perms) => {
				perms.forEach((row) => {
					if(!cache[row.role]) {
						cache[row.role] = [row.permission];
					} else {
						cache[row.role].push(row.permission);
					}
				});
			});
	};

	const module = {

		/**
		 * Returns an item in the cache.
		 */
		get (role) {
			return cache[role];
		},

		/**
		 * Deletes a role
		 */
		remove(role) {
			return knex('role_permission')
				.where('role', role)
				.del()
				.then(() => {
					delete cache[role];
					cacheEventBus.publish({
						type: 'role',
						event: 'remove',
						role
					});
				});
		},

		/**
		 * Updates the database, and sends a message over the event bus for the
		 * other nodes in the cluster to know what is going on.
		 */
		set (role, permissions) {
			return knex.transaction((trx) => {
					return trx
						.select()
						.from('role_permission')
						.where('role', role)
						.then(([role]) => {
							if(role) {
								return replaceRole(trx, role, permissions);
							} else {
								return insertRole(trx, role, permissions);
							}
						})
						.then(trx.commit, trx.rollback);
				});
		},
		/**
		 * Fetches all permissions that match the roles and flattens the array.
		 */
		perms: R.pipe(
			R.chain((role) => cache[role]),
			R.reject(R.isNil),
			R.uniq)

	};

	return loadModule().thenReturn(module);
});
