'use strict';

/**
 * Tokens are used to give temporary session access. Fetches then caches
 * the token only when it needs it the first time.
 *
 * Session tokens cannot be modified, only created or removed.
 */

import procnet from 'procnet';
import B from 'bluebird';
import LRU from 'lru-cache';
import config from '../config';

const cacheSize = config.cache.token.size;
const maxAge = config.session.maxAge;

export default procnet.injectable((sessionTokenStore, cacheEventBus) => {
	const cache = LRU({ max: cacheSize, maxAge: maxAge });

	cacheEventBus.on('clear:session-token', (msg) => {
		cache.dump().forEach(({k, v}) => {
			if(v === msg.userId) {
				cache.del(k);
			}
		});
	});

	const module = {

		/**
		 * @return {Promise} Since in some cases it may require to pull from the
		 * database, otherwise immediately returns a resolved promise with the
		 * user id.
		 */
		get(token) {
			if(!cache.has(token)) {
				return sessionTokenStore
					.find(token) 
					.then((userId) => {
						if(!userId) {
							return;
						}
						cache.set(token, userId);
						return userId;
					});
			}
			return B.resolve(cache.get(token));
		},
		create(userId) {
			return sessionTokenStore
				.create(userId)
				.tap((token) => {
					cache.set(token, userId);
				});
		},
		/**
		 * In case the user wants to log out of all sessions.
		 */
		clear(userId) {
			return sessionTokenStore
				.clear(userId)
				.then(() => {
					cacheEventBus.publish({
						type: 'session-token',
						event: 'clear',
						userId
					});
				});
		}

	};

	return module;
});
