'use strict';

import conf from '../config';
import procnet from 'procnet';
import LRU from 'lru-cache';
import B from 'bluebird';

const maxSize = conf.cache.user.size;

export default procnet.injectable((userStore, cacheEventBus) => {
	const cache = LRU(maxSize);

	cacheEventBus.on('remove:user-role', (msg) => {
		cache.del(msg.userId);
	});

	cacheEventBus.on('update:user-role', (msg) => {
		if(cache.has(msg.userId)) {
			cache.set(msg.userId, msg.roles);
		}
	});

	return {
		get(userId) {
			const rl = cache.get(userId);
			if(!rl) {
				return userStore
					.getRoles(userId)
					.tap((roles) => {
						if(roles) {
							cache.set(userId, roles);
						}
					});
			} else {
				return B.resolve(rl);
			}
		},
		set(userId, roles) {
			cache.set(userId, roles);
			return userStore
				.setRoles(userId, roles)
				.then(() => {
					cacheEventBus.publish({
						type: 'user-role',
						event: 'update',
						userId,
						roles
					});
				});
		},
		/**
		 * This ONLY updates the caches, not the db.
		 */
		remove(userId) {
			cacheEventBus.publish({
				type: 'user-role',
				event: 'remove',
				userId
			});

			cache.del(userId);
		}
	};

});
