'use strict';

/**
 * Templates are loaded into memory lazily. Initial load is slow, but it
 * speeds up considerably afterwards. For each email template there is a text 
 * version and an html one. This is due to how email clients might prefer to 
 * use text versions instead, so it is good practice to send two versions of
 * the same email.
 * Thankfully, I won't have that many so these shouldn't be memory hogs.
 * If a REST api for these is provided, I will need to make sure it doesn't
 * load into memory since I won't have as much control over what the user does
 * with the functionality.
 */

import procnet from 'procnet';
import fs from 'fs';
import conf from '../config';
import path from 'path';
import hogan from 'hogan.js';

export default procnet.injectable(() => {
	const cache = {};
	const dir = conf.mail.templateDir;
	const loadFile = (fileName) => {
		return fs.readFileSync(path.join(dir, fileName)).toString();
	};
	return (name, vars) => {
		if(!cache[name]) {
			// This is fine since it only occurs on the initial load.
			const html = loadFile(name + '.txt.hogan');
			const text = loadFile(name + '.html.hogan');
			const subject = loadFile(name + '.subject.hogan');
			cache[name] = {
				text: hogan.compile(text),
				html: hogan.compile(html),
				subject: hogan.compile(subject)
			};
		}

		return {
			text: cache[name].text.render(vars),
			html: cache[name].html.render(vars),
			subject: cache[name].subject.render(vars)
		};
	};
});
