'use strict';

/**
 * Configuration is only for things which should only change at most
 * once per application restart.
 *
 * There must be an environment variable for each of these.
 */

import R from 'ramda';
import path from 'path';

const defaultConStr = 'postgres://dyna_table:dyna_table@localhost/dyna_table';

const isDev = process.env.NODE_ENV === 'development';

const env = (key) => process.env['DYNATBL_' + key];

const config = {
	db: {
		client: env('DB_CLIENT') || 'pg',
		connection: env('DB_CON_STR') || defaultConStr
	},
	amqp: env('AMQP_URL') || 'amqp://localhost',
	app: {
		port: env('APP_PORT') || '8080'
	},
	cache: {
		token: {
			size: env('CACHE_SESSION_TOKEN') || 20000 // 20k
		},
		user: {
			size: env('CACHE_ROLES') || 10000 // 10k
		}
	},
	session: {
		// default: 1 month per session
		maxAge: env('SESSION_AGE') || 1000 * 60 * 60 * 24 * 30 
	},
	security: {
		// App Token...
		secret: env('APP_SECRET') || isDev ? 'shhh, secret' : ''
	},
	log: {
		file: env('LOG_FILE') || false,
		level: env('LOG_LEVEL') || 'debug'
	},
	mail: (() => {
		const type = env('MAIL_TYPE') || 'smtp';
		const defaultSmtpPort = isDev ? 4650 : 465;
		const defaultPort = type === 'smtp' ? defaultSmtpPort : 443;
		const port = env('MAIL_PORT') || defaultPort;
		const defaultTemplateDir = path.resolve(path.join(__dirname, '../mail'));
		console.log('templateDir:', defaultTemplateDir);
		return {
			// two options: smtp or http-form (e.g., for mailgun api)
			type, 
			// defaults to dev values for now
			port,
			user: env('MAIL_USER') || 'foobar',
			pass: env('MAIL_PASSWD') || '123',
			host: env('MAIL_HOST') || 'localhost',
			sender: env('MAIL_SENDER') || 'noreply@dyna-table.com',
			rewriteRecipient: env('MAIL_RECIPIENT_REWRITE') || false,
			templateDir: env('MAIL_TEMPLATE_DIR') || defaultTemplateDir
		};
	})()
};

// Configurations in this file are essential, they must be speficied.
const validateConfig = (namespace, conf) => {
	const mkName = (namespace, k) => {
		return namespace + (namespace.length === 0 ? '' : '.') + k;
	};
	R.toPairs(conf).forEach(([k, v]) => {
		if(R.isNil(v)) {
			throw new Error('Config ' + mkName(namespace, k) + ' must be defined');
		}
		if(typeof v === 'object') {
			validateConfig(mkName(namespace, k), v);
		} else if(v.length === 0) {
			throw new Error('Config ' + mkName(namespace, k) + ' must be specified');	
		}
	});
};

validateConfig('', config);

export default config;
