'use strict';

/**
 * These are actual form entities used to store data. The types/validations 
 * and other metadata is store on form_metadata.
 */

import uuid from 'uuid';
import procnet from 'procnet';
import R from 'ramda';
import fs from 'fs';
import {viewToDataType} from '../constants';

export default procnet.injectable((knex) => {
	const uniqSytemNames = R.pipe(R.map(R.prop('form_system_name')), R.uniq);

	const proxyObjs = (formMetadata, formProxy) => {
		return formMetadata
			.filter(R.eqProps('form_type', 'proxy'))
			.map(({form_system_name, form_name}) => {
				const fields = formProxy
					.filter(R.eqProps('form_system_name', form_system_name));
				const parent = fields[0].form_parent;
				return {
					parent,
					systemName: form_system_name,
					name: form_name,
					fields: fields.map(R.prop('form_field'))
				};
			});
	};

	const masterObjs = (formMetadata, formMaster) => {
		const fields = (systemName) => {
			return R.pipe(
				R.filter(R.eqProps('form_system_name', systemName)),
				R.map((row) => {
					return {
						type: row.field_type,
						name: row.field_name, 
						systemname: row.field_system_name
					};
				})
			)(formMaster);
		};
		return formMetadata
			.filter(R.eqProps('form_type', 'master'))
			.map(({form_system_name, form_name}) => {
				return {
					name: form_name,
					systemName: form_system_name,
					fields: fields(form_system_name),
					type: 'master'
				};
			});
	};
	return {
		/**
		 * This will return an object with all of the forms in the system.
		 */
		list () {
			knex('form_master')
				.select()
				.then((formMaster) => {
					return knex('form_metadata')
						.select()
						.then((formMetadata) => {
							return {
								formMaster,
								formMetadata
							};
						});
				})
				.then((aggregate) => {
					return knex('form_proxy')
						.select()
						.then(R.flip(R.assoc('formProxy'))(aggregate));
				})
				.then((aggregate) => {
					const {formMaster, formMetadata, formProxy} = aggregate;
					// now I need to transform these into objects...
					return R.reduce(
						(accu, obj) => R.assoc(obj.systemName, obj, accu),
						{},
						R.concat(
							masterObjs(formMetadata, formMaster), 
							proxyObjs(formMetadata, formProxy)
						)
					);
				});

		},
		/**
		 * Creates a new table.
		 * TODO: add support for float/currency types. I think I'll need an additional
		 * argument for this, which is why I'm using an array of objects.
		 *
		 * @param {String} name: translateable name
		 * @param {String} systemName: db name
		 * @param {Array} columns: columns..
		 */
		createMaster(name, systemName, language, columns) {
			return knex.transaction((tx) => {
				// start by creating the entry in the form_metadata table.
				return tx
					.insert({
						form_system_name: systemName,
						form_name: name,
						form_type: 'master'
					})
					.into('form_metadata')
					.then(() => {
						const entries = R.map((col) => {
							return {
								form_system_name: systemName,
								field_type: col.type,
								field_name: col.name,
								field_system_name: col.systemName
							};
						}, columns);
						return tx.insert(entries).into('form_master');
					})
					.then(() => {
						const entries = R.map((col) => {
							return {
								form_system_name: systemName,
								form_system_field: col.systemName,
								form_laguage: language,
								key: 'name',
								value: col.businessName
							};
						}, columns);
						// insert translations...
						return tx.insert(entries).into('form_translation');
					})
					.then(() => {
						return tx.schema.createSchema(systemName, (table) => {
							columns.forEach((col) => {
								const dataType = viewToDataType[col.type];
								table[dataType](col.name);
							});
						});
					})
					.then(tx.commit, tx.rollback);
			});
		},

		/**
		 * Creates a form proxy.
		 * @param {String} name is the translateable name of the form.
		 * @param {String} systemName is the db name of the proxy.
		 * @param {String} parent is the db name of the master form the proxy
		 * points to.
		 * @param {Array} fields Are the fields that the proxy inherits from the
		 * master form.
		 */
		createProxy(name, systemName, parent, fields) {
			return knex.transaction((tx) => {
				return tx
					.insert({
						form_system_name: systemName,
						form_name: name,
						form_type: 'proxy'
					})
					.into('form_metadata')
					.then(() => {
						const entries = R.map((field) => {
							return {
								form_system_name: systemName,
								form_parent: parent,
								form_field: field
							};
						}, fields);
						return tx.insert(entries).into('form_proxy');
					})
					.then(tx.commit, tx.rollback);
			});
		},

		// TODO
		//alter(previous, columns) {
		//}

		/**
		 * Deletes the table
		 */
		drop(systemName, type) {
			return knex.transaction((tx) => {
				return tx
					.del()
					.from('form_metadata')
					.where('form_system_name', systemName)
					.then(() => {
						if(type === 'proxy') {
							return tx
								.del()
								.from('form_proxy')
								.where('form_system_name', systemName);
						} else {
							return tx
								.del()
								.from('form_master')
								.where('form_system_name', systemName);
						}
					})
					.then(() => {
						return tx.schema.dropTable(systemName);
					})
					.then(tx.commit, tx.rollback);
			});

		}

	};
});
