'use strict';

/**
 * These are functions with NO sanity checking.
 *
 * Some automatically generated fields are placed here. The funct
 */
import bcrypt from '../promise/bcrypt';
import uuid from 'uuid';
import R from 'ramda';
import procnet from 'procnet';

export default procnet.injectable((knex) => {
	return {
		create (name, email, pw, roles = ['default'], active = false) {
			// 8 rounds should be more than enough... Considering how often sites
			// implement proper security.
			return bcrypt.hash(pw, 8).then((password) => {
				const id = uuid.v4();
				const payload = {
					id,
					name,
					email,
					roles: JSON.stringify(roles),
					password,
					active
				};

				return knex('user')
					.insert(payload)
					.thenReturn(payload);
			});
		},
		
		/**
		 * Activates the user
		 */
		activate(user_id) {
			return knex('user')
				.where({ user_id })
				.update({ active: true });
		},

		/**
		 * Finds the user...
		 */
		find(...args) {
			const k = knex('user');
			return k
				.where.apply(k, args)
				.select()
				.then(R.head);
		},

		getRoles(user_id) {
			return knex('user')
				.select('roles')
				.where('id', user_id)
				.then(R.pipe(R.head, R.prop('roles')));
		},

		setRoles(user_id, roles) {
			return knex('user')
				.update('roles', JSON.stringify(roles))
				.where('id', user_id);
		},

		/**
		 * Count the number of failures and update the time to guard against
		 * brute force attacks.
		 */
		failLogin(user_id) {
			return knex.transaction((tx) => {
				return tx
					.select()
					.from('user')
					.where('id', user_id)
					.then(([user]) => {
						return tx
							.from('user')
							.where('id', user_id)
							.update({ 
								last_failed_login: new Date(),
								failed_logins: user.failed_logins + 1
							});
					})
					.then(tx.commit, tx.rollback);
			});
		},

		/**
		 * Makes the fail login counter turn to zero.
		 */
		succeedLogin(user_id) {
			return knex('user')
				.where('id', user_id)
				.update('failed_logins', 0);
		}
	};
});
