'use strict';

import procnet from 'procnet';
import bcrypt from '../promise/bcrypt';
import R from 'ramda';
import error from '../error';

// See:
// https://www.owasp.org/index.php/Authentication_Cheat_Sheet
// #Implement_Proper_Password_Strength_Controls
const optional_complexity = [
	/[A-Z]/,
	/[a-z]/,
	/[0-9]/,
	/[\W]/
];

const passwordErrors = (password) => {
	if(password.length > 20) {
		return;
	}
	if(password.length < 10) {
		return 'password_min_length_failed';
	}

	const passesOptions = optional_complexity.filter((reg) => reg.test(password));
	if(passesOptions.length < 3) {
		return 'password_complexity_failed';
	}
	// jscs:disable
	var lc, c;
	for(var i = 0; i < password.length; i++) {
		c = password[i];
		if(c === lc && c === password[i + 1]) {
			return 'password_maximum_char_repeat_failed';
		}
		lc = c;
	}
	// jscs:enable
	
	// This kind of password format would have to be the most
	// popular in the world. Attackers are aware of this thanks to
	// password leaks in the past. Block this one by default.
	if(/^[A-Z]{1,2}[a-z]+(\d|\W)+$/.test(password)) {
		return 'password_format_is_common';
	}
};

export default procnet.injectable((userStore) => {
	const mod = {};

	/**
	 * Basic email validation.
	 *
	 * @Returns {Promise} of true if the email doesn't exist in the system. 
	 * Promise is rejected if the email is in use.
	 */
	mod.email = (email) => {
		if(!/\S+@\S+/.test(email)) {
			return error('invalid_email_format');
		}
		return userStore
			.find('email', email)
			.then((user) => user ? error('email_in_use') : true);
	};

	/**
	 * Returns a resolved promise if the email and password for the user creation
	 * is valid, failed promise containing a translation token otherwise.
	 */
	mod.create = (name, email, password) => {
		const err = passwordErrors(password);
		if(err) {
			return error(err);
		}
		return mod.email(email);
	};
	/**
	 * Returns an error string if the account is locked out. undefined otherwise.
	 */
	mod.timedout = (user) => {
		// TODO: needs to be configurable
		if(user.failed_logins >= 5) {
			// I need to compute how long its been since the user failed to
			// login.
			const lastLogin = new Date(user.last_failed_login).getTime();
			// TODO: needs to be configurable
			const bruteforceTimeout = 1000 * 60; // one minute timeout
			if(Date.now() - lastLogin < bruteforceTimeout) {
				return true;
			}
		}
		return false;
	};
	/**
	 * Return a rejected promise with an error or a resolve promise with the
	 * user argument passed along and a boolean value in a tuple. If false means
	 * the password does not match.
	 *
	 * @param {String} email
	 * @param {String} password
	 * @param {Object} user
	 * @returns {Promise<Tuple2<Object, Boolean>>}
	 */
	mod.login = R.curry((email, password, user) => {

		if(R.isNil(email)) {
			return error('email_required');
		}

		if(R.isNil(password)) {
			return error('password_required');
		}

		if(R.isNil(user)) {
			return error.notFound('user_not_exist');
		}

		if(mod.timedout(user)) {
			return error('password_retries_maxed');
		}

		if(!user.active) {
			return error('user_not_active');
		}

		return bcrypt
			.compare(password, user.password)
			.then(R.flip(R.append)([user]));
	});

	return mod;
});
